"use strict";
const affReader_1 = require("./affReader");
const fileReader_1 = require("./fileReader");
const Rx = require("rxjs/Rx");
class HunspellReader {
    constructor(affFile, dicFile) {
        this.affFile = affFile;
        this.dicFile = dicFile;
        this.aff = affReader_1.parseAffFileToAff(affFile);
    }
    /**
     * @internal
     */
    readDicEntries(aff) {
        return fileReader_1.lineReader(this.dicFile, aff.affInfo.SET)
            .skip(1) // Skip the first line -- it's the number of words in the file context's.
        ;
    }
    /**
     * @internal
     */
    readDicWords() {
        return Rx.Observable.fromPromise(this.aff)
            .flatMap(aff => this.readDicEntries(aff))
            .map(line => {
            const [word, rules] = line.split('/', 2);
            return { word, rules };
        });
    }
    readWordsRx() {
        const r = Rx.Observable.fromPromise(this.aff)
            .flatMap(aff => this.readDicEntries(aff)
            .flatMap(dicWord => aff.applyRulesToDicEntry(dicWord)));
        return r;
    }
    /**
     * Reads all the word combinations out of a hunspell dictionary.
     */
    readWords() {
        return this.readWordsRx()
            .map(affWord => affWord.word);
    }
    /**
     * Reads the words in the dictionary without applying the transformation rules.
     */
    readRootWords() {
        return this.readDicWords()
            .map(w => w.word);
    }
}
exports.HunspellReader = HunspellReader;
//# sourceMappingURL=HunspellReader.js.map