"use strict";
const chai_1 = require("chai");
const Rx = require("rxjs/Rx");
const trieCompact_1 = require("./trieCompact");
const words = [
    'create',
    'created',
    'creates',
    'creating',
    'creation',
    'creation\'s',
    'creationism',
    'creationism\'s',
    'creationisms',
    'creationist',
    'creationist\'s',
    'creationists',
    'creations',
    'creative',
    'creative\'s',
    'creatively',
    'creativeness',
    'creativeness\'s',
    'creatives',
    'do_not',
    'do~not',
    'do=not',
];
describe('validate trieCompact functions', () => {
    it('tests backSpaceEmitSequenceToLength', () => {
        chai_1.expect(trieCompact_1.backSpaceEmitSequenceToLength('___Hello').length).to.be.equal(3);
        chai_1.expect(trieCompact_1.backSpaceEmitSequenceToLength('___Hello').offset).to.be.equal(3);
        chai_1.expect(trieCompact_1.backSpaceEmitSequenceToLength(trieCompact_1.singleDeleteChar + trieCompact_1.singleDeleteChar).length).to.be.equal(2);
        chai_1.expect(trieCompact_1.backSpaceEmitSequenceToLength(trieCompact_1.multiDeleteChar + '3Hello').length).to.be.equal(3);
        chai_1.expect(trieCompact_1.backSpaceEmitSequenceToLength(trieCompact_1.multiDeleteChar + '3Hello').offset).to.be.equal(2);
    });
    it('tests calcBackSpaceEmit', () => {
        chai_1.expect(trieCompact_1.calcBackSpaceEmit(1)).to.be.equal('_');
        chai_1.expect(trieCompact_1.calcBackSpaceEmit(2)).to.be.equal(trieCompact_1.multiDeleteChar + '2');
        chai_1.expect(trieCompact_1.calcBackSpaceEmit(5)).to.be.equal(trieCompact_1.multiDeleteChar + '5');
        chai_1.expect(trieCompact_1.calcBackSpaceEmit(129)).to.be.equal(trieCompact_1.multiDeleteChar + 'p' + trieCompact_1.multiDeleteChar + 'p_');
    });
    it('tests trieCompactSortedWordList', () => {
        return trieCompact_1.trieCompactSortedWordList(Rx.Observable.from(words))
            .toArray()
            .toPromise().then(values => {
            const str = values.join('');
            const expected = 'create\nd_s=2ing=';
            const allWords = words.join('\n');
            // const json = JSON.stringify(str);
            chai_1.expect(str.slice(0, expected.length)).to.be.equal(expected);
            chai_1.expect(str.length).to.be.lessThan(allWords.length);
        });
    });
    it('makes sure a compacted trie can be extracted', () => {
        const stream = trieCompact_1.trieCompactSortedWordList(Rx.Observable.from(words));
        return trieCompact_1.trieCompactExtract(stream)
            .toArray()
            .toPromise().then(values => {
            chai_1.expect(values).to.be.deep.equal(words);
        });
    });
    it('tests escaping letters', () => {
        const tests = [
            ['this_is_a~test', 'this~_is~_a~~test'],
            ['this=is_a~test', 'this~=is~_a~~test'],
        ];
        tests.forEach(([before, after]) => {
            const r = trieCompact_1.escapeLetters(before);
            chai_1.expect(r).to.be.equal(after);
            const s = trieCompact_1.unescapeLetters(r);
            chai_1.expect(s).to.be.equal(before);
        });
    });
});
//# sourceMappingURL=trieCompact.test.js.map